@@ -6,7 +6,12 @@ except:
     import pickle
 import weakref
 import warnings
-from lucenequerybuilder import Q
+try:
+    from lucenequerybuilder import Q
+except:
+    class Q(object):
+        def __init__(self, *args, **kwargs):
+            raise ImportError("Try installing lucene-querybuilder first.")
 
 from neo4jrestclient import options
 from neo4jrestclient.constants import (
@@ -85,7 +90,7 @@ class GraphDatabase(object):
         if response_json:
             self._relationship_index = response_json['relationship_index']
             self._node = response_json['node']
-            self._labels = response_json.get('labels',
+            self._labels = response_json.get('node_labels',
                                              "{}labels".format(self.url))
             self._labels_list = None
             self._node_index = response_json['node_index']
@@ -186,7 +191,7 @@ class GraphDatabase(object):
             globals()[options.TX_NAME] = self._transactions[transaction_id]
         return self._transactions[transaction_id]
 
-    def query(self, q, params=None, returns=RAW, tx=None):
+    def query(self, q, params=None, returns=RAW, data_contents=None, tx=None):
         if self._cypher or self._transaction:
             types = {
                 "node": Node,
@@ -201,8 +206,14 @@ class GraphDatabase(object):
                     and self.VERSION and self.VERSION.split(".")[0] >= "2"):
                 tx = self.transaction(for_query=True, execute=True,
                                       using_globals=False)
-            return QuerySequence(self._cypher, self._auth, q=q, params=params,
-                                 types=types, returns=returns, tx=tx)
+            query_sequence = QuerySequence(
+                self._cypher, self._auth, q=q, params=params,
+                types=types, returns=returns, data_contents=data_contents,
+                tx=tx
+            )
+            if tx is not None and tx.id in self._transactions:
+                del self._transactions[tx.id]
+            return query_sequence
         else:
             raise CypherException
 
@@ -414,7 +425,7 @@ class TransactionOperationProxy(dict, object):
                                               "self", auth=_auth)
                     else:
                         self._proxy = cls(Path, data["body"], auth=_auth)
-        else:
+        elif data and "body" in data:
             if "self" in data["body"] and data["body"]["self"] != url:
                 self._proxy = cls(data["body"]["self"],
                                   update_dict=data["body"], auth=_auth)
@@ -429,6 +440,13 @@ class TransactionOperationProxy(dict, object):
         _body = dict.__getitem__(self, "body")
         return "{%s}" % _body["id"]
 
+    def delete(self, tx=None):
+        url = "{{{0}}}".format(object.__getattribute__(self, "_job_id"))
+        tx = Transaction.get_transaction(tx)
+        if tx:
+            return tx.append(TX_DELETE, url, obj=self)
+        return None
+
     # Node functions
     def _get_relationships(self):
         """
@@ -610,6 +628,7 @@ class Transaction(object):
                 ref_object = referenced_object()
                 result = results[ref_object()["id"]]
                 if "returns" in result:
+                    cls, url = None, None
                     if "location" in result:
                         cls = result["returns"]
                         url = result["location"]
@@ -624,6 +643,9 @@ class Transaction(object):
                             url = None
                             ref_object.change(cls.__class__, cls, data=result,
                                               auth=auth)
+                    elif result["returns"] is None:
+                        # Weak ref is not that easy to change
+                        ref_object = None
                     if cls and url:
                         ref_object.change(cls, url, data=result, auth=auth)
         self.references = []
@@ -670,7 +692,7 @@ class Transaction(object):
                                                               typ=returns,
                                                               **params)
             self.operations.append(transaction_operation)
-            if method in (TX_POST, TX_GET):
+            if method in (TX_POST, TX_GET, TX_DELETE):
                 self.references.append(weakref.ref(transaction_operation))
         return transaction_operation
 
@@ -1172,6 +1194,11 @@ class Node(Base):
                 'language': 'javascript',
                 'body': 'false',
             }})
+        elif stop:
+            data.update({'prune_evaluator': {
+                'language': 'javascript',
+                'body': stop,
+            }})
         if returnable in (RETURN_ALL_NODES, RETURN_ALL_BUT_START_NODE):
             data.update({"return_filter": {
                 "language": "builtin",
