@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 __author__ = "Javier de la Rosa"
 __license__ = "GPL 3"
-__version__ = "2.0.4"
+__version__ = "2.1.0"
 __email__ = "versae@gmail.com"
 __url__ = "https://github.com/versae/neo4j-rest-client"
 __description__ = """Object-oriented Python library to interact with """ \
@@ -50,3 +50,6 @@ TX_GET = "GET"
 TX_PUT = "PUT"
 TX_POST = "POST"
 TX_DELETE = "DELETE"
+# Queries REST data contents
+DATA_GRAPH = "graph"
+DATA_ROWS = "row"
