@@ -7,9 +7,10 @@ from collections import Sequence
 import warnings
 
 from neo4jrestclient.constants import RAW
+from neo4jrestclient.iterable import Iterable
 from neo4jrestclient.request import Request
 from neo4jrestclient.exceptions import StatusException, TransactionException
-from neo4jrestclient.utils import text_type, string_types, in_ipnb
+from neo4jrestclient.utils import text_type, string_types, in_ipnb, rewrites
 
 
 class BaseQ(object):
@@ -298,10 +299,11 @@ class CypherException(Exception):
 class QuerySequence(Sequence):
 
     def __init__(self, cypher, auth, q, params=None, types=None, returns=None,
-                 lazy=False, tx=None):
+                 lazy=False, data_contents=None, tx=None):
         self.q = q
         self.params = params
         self.columns = None
+        self.stats = None
         self._skip = None
         self._limit = None
         self._order_by = None
@@ -315,9 +317,10 @@ class QuerySequence(Sequence):
         # Store rows and graphs information
         self._elements_row = None
         self._elements_graph = None
+        self._data_contents = data_contents
         if tx:
             tx.append(q=self.q, params=self.params, returns=self._returns,
-                      obj=self)
+                      data_contents=data_contents, obj=self)
             tx.execute()
         elif not lazy:
             self._get_elements()
@@ -330,11 +333,20 @@ class QuerySequence(Sequence):
                     self, elements=response["data"], returns=self._returns
                 )
                 self.columns = response.get("columns", None)
+                self.stats = response.get("stats", None)
             except:
                 self._elements = response
         return self._elements
     elements = property(_get_elements)
 
+    def _get_rows(self):
+        return self._elements_row
+    rows = property(_get_rows)
+
+    def _get_graph(self):
+        return self._elements_graph
+    graph = property(_get_graph)
+
     def __getitem__(self, key):
         return self.elements[key]
 
@@ -690,20 +702,66 @@ class QuerySequence(Sequence):
         if cypher is None:
             cypher = cls._cypher
         neutral = lambda x: x
-        # For IPython Notebook
-        cls._elements_row = []
-        cls._elements_graph = []
+
+        def cast_element(element, func):
+            # We also allow the use of constants like NODE, etc
+            if isinstance(func, string_types):
+                func_lower = func.lower()
+                if func_lower in types_keys:
+                    func = types[func_lower]
+            if func in (types.get("node", ""),
+                        types.get("relationship", "")):
+                obj = func(element["self"], update_dict=element,
+                           auth=auth, cypher=cypher)
+                return obj
+            elif func in (types.get("path", ""),
+                          types.get("position", "")):
+                obj = func(element, auth=auth, cypher=cypher)
+                return obj
+            elif func in (None, True, False):
+                sub_func = lambda x: x is func
+                return sub_func(element)
+            elif isinstance(element, (list, tuple)):
+                if isinstance(func, Iterable):
+                    objs = [cast_element(obj, func._class) for obj in element]
+                    return iter(objs)
+                elif isinstance(func, (list, tuple)):
+                    objs = []
+                    for i, obj in enumerate(element):
+                        try:
+                            sub_func = func[i]
+                        except IndexError:
+                            pass  # We keep the last value for sub_func
+                        objs.append(cast_element(obj, sub_func))
+                    return type(func)(objs)
+                elif callable(func):
+                    return func(element)
+                else:
+                    objs = [cast_element(obj, func) for obj in element]
+                    return type(func)(objs)
+            elif func == RAW:
+                return element
+            else:
+                return func(element)
+
+        def _process_data_contents(element):
+            if "row" in element:
+                if cls._elements_row is None:
+                    cls._elements_row = []
+                cls._elements_row.append(rewrites(element["row"]))
+            if "graph" in element:
+                if cls._elements_graph is None:
+                    cls._elements_graph = []
+                cls._elements_graph.append(rewrites(element["graph"]))
+
         if not returns or returns is RAW:
             if len(elements) > 0:
                 results = []
                 for element in elements:
-                    # For IPython Notebook
-                    if "row" in element:
-                        cls._elements_row.append(element["row"])
-                    if "graph" in element:
-                        cls._elements_graph.append(element["graph"])
+                    # For IPython Notebook and data_contents
+                    _process_data_contents(element)
                     # For transactional Cypher endpoint
-                    results.append(element.get("rest", None))
+                    results.append(rewrites(element.get("rest", None)))
                 return results
             else:
                 return elements
@@ -714,14 +772,12 @@ class QuerySequence(Sequence):
             else:
                 returns = list(returns)
             for row in elements:
-                # For IPython Notebook
-                if "row" in row:
-                    cls._elements_row.append(row["row"])
-                if "graph" in row:
-                    cls._elements_graph.append(row["graph"])
+                # For IPython Notebook and data_contents
+                _process_data_contents(row)
                 # For transactional Cypher endpoint
                 if "rest" in row:
                     row = row["rest"]
+                row = rewrites(row)
                 # We need both list to have the same lenght
                 len_row = len(row)
                 len_returns = len(returns)
@@ -732,28 +788,8 @@ class QuerySequence(Sequence):
                 casted_row = []
                 types_keys = types.keys()
                 for i, element in enumerate(row):
-                    # if "rest" in element:
-                    #     element = element["rest"]
                     func = returns[i]
-                    # We also allow the use of constants like NODE, etc
-                    if isinstance(func, string_types):
-                        func_lower = func.lower()
-                        if func_lower in types_keys:
-                            func = types[func_lower]
-                    if func in (types.get("node", ""),
-                                types.get("relationship", "")):
-                        obj = func(element["self"], update_dict=element,
-                                   auth=auth, cypher=cypher)
-                        casted_row.append(obj)
-                    elif func in (types.get("path", ""),
-                                  types.get("position", "")):
-                        obj = func(element, auth=auth, cypher=cypher)
-                        casted_row.append(obj)
-                    elif func in (None, True, False):
-                        sub_func = lambda x: x is func
-                        casted_row.append(sub_func(element))
-                    else:
-                        casted_row.append(func(element))
+                    casted_row.append(cast_element(element, func))
                 if cls is not None and cls._return_single_rows:
                     results.append(*casted_row)
                 else:
@@ -855,18 +891,27 @@ class QueryTransaction(object):
                     obj, elements=result["data"], returns=returns
                 )
                 obj.columns = result.get("columns", None)
+                obj.stats = result.get("stats", None)
                 results.append(obj)
             return results
         else:
             return result_list
 
-    def append(self, q, params=None, returns=None, obj=None):
+    def append(self, q, params=None, returns=None, obj=None,
+               data_contents=None):
         result_data_contents = ["REST"]
-        if in_ipnb():
+        if in_ipnb() or data_contents is True:
             result_data_contents += ["row", "graph"]
+            include_stats = True
+        elif data_contents and data_contents.lower() in ["row", "graph"]:
+            include_stats = True
+            result_data_contents.append(data_contents.lower())
+        else:
+            include_stats = False
         statement = {
             "statement": q,
             "parameters": params,
+            "includeStats": include_stats,
             "resultDataContents": result_data_contents,
         }
         self.statements.append(statement)
@@ -916,6 +961,7 @@ class QueryTransaction(object):
                     if obj is not None:
                         obj._elements = []
                         obj.columns = None
+                        obj.stats = None
                 self.executed = []
             else:
                 raise TransactionException(response.status_code)
